import pandas
from pandas import DataFrame
from tqdm.auto import tqdm
import schema
from .util import groups_of
from schema.model import Project, ScienceProduct, AncillaryProduct, File
from sqlalchemy.sql import *
from schema.legacy_model import *
from schema.ngasmodel import NGASFile


def main():
    session = pyat.schema.create_session('SDM', profile='local')
    ngas = pyat.schema.create_session('NGAS', profile='local')
    legacy = pyat.schema.create_session('LEGACY', profile='local')

    print('loading SDMs from NGAS')
    sdm_query = select([NGASFile.file_id.label('ngas_id')])\
        .where(NGASFile.format == 'application/x-sdm')\
        .order_by(NGASFile.file_id)
    ngas_sdms = pandas.read_sql(sdm_query, ngas.connection())

    print('loading SDMs from new archive')
    ngas_query = select([File.ngas_id]).where(File.format == 'sdm').order_by(File.ngas_id)
    archive_sdms = pandas.read_sql(ngas_query, session.connection())

    # mark them as seen
    archive_sdms['present'] = True

    # left join to find the missing files
    result = ngas_sdms.set_index('ngas_id').join(archive_sdms.set_index('ngas_id'))
    missing = result[result.present.isna()]

    # with each of the missing files, we need to find their metadata in the legacy archive
    # unfortunately, Oracle will only let us have up to 1000 items in an IN() clause, so
    # we must do this in batches
    count = 1000
    matching = DataFrame([], columns=['file_id', 'project_code', 'sb_id', 'eb_id'])
    print('matching missing SDMs with legacy archive metadata')

    # regrettably, this is about the fastest thing the script does,
    # and it's the only part we can get a progress bar on
    for group in tqdm(groups_of(missing, count), total=len(missing)/count):
        matching = matching.append(pandas.read_sql(
            # I do not find this to be especially more readable than SQL, but there is
            select([t_ngas_file_sets.c.file_id.distinct(),
                    t_ngas_file_sets.c.file_set_id,
                    t_archive.c.project_code,
                    t_archive.c.sb_id,
                    t_archive.c.eb_id,
                    # this case expression gets us the filename that John would write the file to
                    case([(t_ngas_file_sets.c.file_id.like('%.sdm'),
                           func.replace(t_ngas_file_sets.c.entity_type_name, 'Table', '') + '.xml'),
                          (t_ngas_file_sets.c.file_id.like('%.bin'),
                           func.replace(t_ngas_file_sets.c.entity_type_name, 'Table', '') + '.bin'),
                          (t_ngas_file_sets.c.file_id.like('%.bdf'),
                           func.replace(t_ngas_file_sets.c.file_id, '.bdf', ''))]).label('filename')])
                .where(and_(t_archive.c.arch_file == t_ngas_file_sets.c.file_set_id,
                            t_ngas_file_sets.c.file_id.in_(group.index)))
                .order_by(t_ngas_file_sets.c.file_id),
            legacy.connection()))

    matching.to_csv('output3.csv')
