import json
import logging

from pycapo import CapoConfig
from pyramid.view import view_config

from . import ResponseMsg

logger = logging.getLogger(__name__)


@view_config(route_name='get_rh_url', renderer='jsonp')
def get_rh_url(request):
    r"""Get the base Request Handler url.
    :param request: the capo_profile should be in the request.session.
    :return: a JSON blob with the RH url and a response message indicating success or failure.
    """
    c = CapoConfig(profile=request.session['capo_profile'])
    try:
        rh_url = c.get('edu.nrao.archive.archiveIface.rhBaseUrl')
        response_msg = ResponseMsg(True, ResponseMsg.SUCCESS, 'Request processed')
    except KeyError:
        rh_url = None
        response_msg = ResponseMsg(False, ResponseMsg.HARD_ERROR,
                                   'capo key: edu.nrao.archive.archiveIface.rhBaseUrl, not found')

    return json.dumps(dict(rh_url=rh_url,
                           response_msg=response_msg.as_dict()))


@view_config(route_name='get_release_notes_url', renderer='jsonp')
def get_release_notes_url(request):
    r"""Get the Release Notes url.
    :param request: the capo_profile should be in the request.session.
    :return: a JSON blob with the Release Notes url and a response message indicating success or failure.
    """
    c = CapoConfig(profile=request.session['capo_profile'])
    try:
        release_notes_url = c.get('edu.nrao.archive.releaseNotesLink')
        response_msg = ResponseMsg(True, ResponseMsg.SUCCESS, 'Request processed')
    except KeyError:
        release_notes_url = None
        response_msg = ResponseMsg(False, ResponseMsg.HARD_ERROR,
                                   'Failed to find release notes url')

    return json.dumps(dict(release_notes_url=release_notes_url,
                           response_msg=response_msg.as_dict()))


@view_config(route_name='get_application_parameters', renderer='jsonp')
def get_application_parameters(request):
    c = CapoConfig(profile=request.session['capo_profile'])
    download_size_limit = 10000000000000  # 10TB
    download_obs_limit = 10
    download_image_limit = 20
    try:
        download_size_limit = c.get('edu.nrao.archive.archiveIface.downloadSizeLimit')
        download_obs_limit = c.get('edu.nrao.archive.archiveIface.downloadObsLimit')
        download_image_limit = c.get('edu.nrao.archive.archiveIface.downloadImageLimit')
        response_msg = ResponseMsg(True, ResponseMsg.SUCCESS, 'Request processed')
    except KeyError:
        response_msg = ResponseMsg(False, ResponseMsg.HARD_ERROR,
                                   'Failed to find application parameters')

    return json.dumps(dict(download_size_limit=download_size_limit,
                           download_obs_limit=download_obs_limit,
                           download_img_limit=download_image_limit,
                           response_msg=response_msg.as_dict()))


@view_config(route_name='get_pre_release_feature_flag', renderer='jsonp')
def get_pre_release_feature_flag(request):
    c = CapoConfig(profile=request.session['capo_profile'])
    # We default the pre-release feature flag to true
    pre_release_flag = True
    try:
        capo_pre_release_str = c.get('edu.nrao.archive.archiveIface.preReleaseFeature')
        pre_release_flag = False if capo_pre_release_str == 'false' else True
        response_msg = ResponseMsg(True, ResponseMsg.SUCCESS, 'Request processed')
    except KeyError:
        response_msg = ResponseMsg(False, ResponseMsg.HARD_ERROR,
                                   'Failed to find application parameters')

    return json.dumps(dict(pre_release_flag=pre_release_flag,
                           response_msg=response_msg.as_dict()))


@view_config(route_name='get_show_all_thumbnails_flag', renderer='jsonp')
def get_show_all_thumbnails_flag(request):
    c = CapoConfig(profile=request.session['capo_profile'])
    show_all_thumbnails = False
    try:
        show_all_thumbnails_str = c.get('edu.nrao.archive.archiveIface.showAllThumbnails')
        show_all_thumbnails = True if show_all_thumbnails_str == 'true' else False
        response_msg = ResponseMsg(True, ResponseMsg.SUCCESS, 'Request processed')
    except KeyError:
        response_msg = ResponseMsg(False, ResponseMsg.HARD_ERROR,
                                   'Failed to find application parameters')

    return json.dumps(dict(show_all_thumbnails=show_all_thumbnails,
                           response_msg=response_msg.as_dict()))


