import os

from pycapo import CapoConfig
from pyramid.events import subscriber, NewRequest

import logging
log = logging.getLogger(__name__)


@subscriber(NewRequest)
def session_service(event):
    if 'current_user' not in event.request.session:
        current_user = event.request.environ.get('REMOTE_USER', None)
        event.request.session['current_user'] = current_user
    else:
        current_user = None
        event.request.session['current_user'] = current_user

    if 'capo_profile' not in event.request.session:
        capo_profile = event.request.environ.get('CAPO_PROFILE', os.environ['CAPO_PROFILE'])
        event.request.session['capo_profile'] = capo_profile
    else:
        capo_profile = event.request.session['capo_profile']

    if 'acknowledgement' not in event.request.session:
        c = CapoConfig(profile=capo_profile)
        # This might seem backwards, its mostly for readability, if we are showing a splash according to capo, we do
        # not want to "pre-acknowledge".  The acknowledgement is a "the user has read the splash" and dismissed it.
        pre_acknowledge = False if c.get('edu.nrao.archive.archiveIface.displaySplashScreenMessage') == 'true' else True
        event.request.session['acknowledgement'] = pre_acknowledge

    def cors_headers(request, response):
        response.headers.update({
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Methods': 'POST,GET,OPTIONS',
            'Access-Control-Allow-Headers': 'Origin, Content-Type, Accept, Authorization',
            'Access-Control-Allow-Credentials': 'false',
            'Access-Control-Max-Age': '1728000',
        })
    event.request.add_response_callback(cors_headers)
