from __future__ import print_function
import logging
import json
from pyramid.view import view_config
from sqlalchemy import and_

from . import def_name
from services.solr.query_builder import ArchiveSolrParameters
from services.client_services import ResponseMsg

from pyat.schema import AlmaSourceDatum, AlmaSpwDatum, t_alma_spw_sources
from services import DB

log = logging.getLogger(__name__)


@view_config(route_name='mous_details_view', renderer='jsonp')
def mous_details_view(request):
    details = []
    try:
        [sources, windows, binder] = get_details(**request.params)
        fields = [dict(alma_ous_id=s.alma_ous_id,
                       field_name=s.field_name,
                       ra=s.ra,
                       dec=s.dec,
                       integration_time=s.integration_time,
                       angular_resolution=s.angular_resolution,
                       largest_angular_scale=s.largest_angular_scale,
                       science_field=s.science_field) for s in sources if s.science_field is True]
        # The spectral windows seems to have the frequencies in GHz, bandwidth in Hz, and spectral res in kHz,
        # normalizing them all to Hz on the way out for consistency.
        spws = [dict(alma_ous_id=w.alma_ous_id,
                     spw_name=w.spw_name,
                     min_frequency=float(w.min_frequency) * 1000000000,
                     max_frequency=float(w.max_frequency) * 1000000000,
                     bandwidth=float(w.bandwidth),
                     num_channels=w.num_channels,
                     spectral_resolution=w.spectral_resolution * 1000) for w in windows]
        joiner = [dict(alma_ous_id=b.alma_ous_id,
                       spw_name=b.spw_name,
                       field_name=b.field_name) for b in binder]
        details = dict(fields=fields,
                       spws=spws,
                       joiner=joiner)
        response_msg = ResponseMsg(True, ResponseMsg.SUCCESS, 'Request processed')
    except Exception as ex:
        ex_msg = 'Exception caught in {0}.{1}: {2}'.format(__name__, def_name(), ex)
        log.info(ex_msg)
        response_msg = ResponseMsg(False, ResponseMsg.HARD_ERROR, ex_msg)
    return json.dumps(dict(details=details,
                           response_msg=response_msg.as_dict()))


def get_details(**params):
    try:
        queryParams = ArchiveSolrParameters(**params)
        sources = DB['SDM'].query(AlmaSourceDatum) \
            .filter(AlmaSourceDatum.alma_ous_id == queryParams.mousId) \
            .all()
        spws = DB['SDM'].query(AlmaSpwDatum) \
            .filter(AlmaSpwDatum.alma_ous_id == queryParams.mousId)\
            .order_by(AlmaSpwDatum.min_frequency) \
            .all()
        binder = DB['SDM'].query(t_alma_spw_sources) \
            .filter(t_alma_spw_sources.c.alma_ous_id == queryParams.mousId) \
            .all()
        return [sources, spws, binder]
    except Exception as x:
        ex = 'Exception caught in {0}: {1}'.format(def_name(), str(x))
        raise Exception(ex)
